import datetime

from argparse import ArgumentParser, ArgumentTypeError

def valid_date(s):
        try:
                return datetime.datetime.strptime(s, "%Y-%m-%d")
        except ValueError:
                raise ArgumentTypeError("Not a valid date: '{0}'.".format(s))

parser = ArgumentParser()
parser.add_argument('wrf_data_fpath_pattern', type=str,        help='WRF input data path pattern (including strptime pattern for date and {domain:s} for domain (01, 02, ...))')
parser.add_argument('date',                   type=valid_date, help='Date to process (YYYY-mm-dd)')
parser.add_argument('interval_hours',         type=int,        help='Time interval in hours for plotting')
parser.add_argument('fcst_time_hours',        type=int,        help='Forecast time in hours')
parser.add_argument('plot_fpath_pattern',     type=str,        help='Plot output path pattern (including strptime pattern for date and {plot:s} for plot identifier (e.g., 500hPa_europe...)')

args = parser.parse_args()

from plots import plevel_map, sfc_map, slp_map
import netCDF4
from wrf import getvar

times = [ args.date + datetime.timedelta(hours=t) for t in range(0, args.fcst_time_hours, args.interval_hours) ]

# for calculating delta precip
prec_prev = None

for t in times:
        input_path_d01   = t.strftime(args.wrf_data_fpath_pattern).format(domain=1)

        try:
                nc_d01      = netCDF4.Dataset(input_path_d01)
        except Exception as e:
                print("Not all data available for step {:s}: {:s}".format(t.strftime("%Y-%m-%d %H"), str(e)))
                continue

        print(t)

        pois_southern_germany = [   { 'name':'Munich',     'lon':11.582,    'lat':48.135,    'ms':5,   'fontsize':9 }, 
                                        { 'name':'Augsburg',   'lon':10.894446, 'lat':48.366512, 'ms':2.5, 'fontsize':8 },
                                        { 'name':'Regensburg', 'lon':12.101624, 'lat':49.013432, 'ms':2.5, 'fontsize':8 },
                                        { 'name':'Ingolstadt', 'lon':11.426311, 'lat':48.761423, 'ms':2.5, 'fontsize':8 },
                                        { 'name':'Innsbruck',  'lon':11.388397, 'lat':47.258320, 'ms':2.5, 'fontsize':8 }
                                        ]

        # 500 hPa Europe domain
        output_path = t.strftime(args.plot_fpath_pattern).format(plot='500hPa_Europe')
        plevel_map(t, nc_d01, output_path, 500)

        # Surface Europe domain
        # temperature
        sfc_map(t, nc_d01, lambda nc: getvar(nc, 'T2') - 273.15, 
                levels=[-25, -20, -15, -12.5, -10, -7.5, -5, -2.5, 0, 2.5, 5, 7.5, 10, 12.5, 15, 17.5, 20, 22.5, 25, 27.5, 30, 32.5, 35],
                ticks=[-25, -15, -10, -5, 0, 5, 10, 15, 20, 25, 30, 35],
                label="Temperature (2m) [°C]", pois={}, 
                output_path=t.strftime(args.plot_fpath_pattern).format(plot='T_Europe'))
        # Ozone
        sfc_map(t, nc_d01, lambda nc: getvar(nc, 'o3')[0,:,:] * 1e3, 
                levels=range(0, 160, 10),
                ticks=range(0, 150, 20),
                label="Ozone (ground level) [ppbv]", pois={}, 
                output_path=t.strftime(args.plot_fpath_pattern).format(plot='O3_Europe'),
                extend='max')
        # NO2
        sfc_map(t, nc_d01, lambda nc: getvar(nc, 'no2')[0,:,:] * 1e3, 
                levels=range(0, 110, 10),
                ticks=range(0, 110, 20),
                label="NO$_2$ (ground level) [ppbv]", pois={}, 
                output_path=t.strftime(args.plot_fpath_pattern).format(plot='NO2_Europe'),
                extend='max')
        # PM
        def get_pm(nc, bins=[ 1, 2, 3 ]) :
                pmspecs = [ "so4", "no3", "smpa", "smpbb", "glysoa_sfc", "biog1_c", "biog1_o", "cl", "co3", "nh4", "na", "cl", "oin", "oc", "bc", "water" ] # ug kg-1
                alt     = getvar(nc, "ALT")[0,:,:] # inverse density m3 kg-1
                all     =  [ getvar(nc, spec + "_a{:02d}".format(bin))[0,:,:] / alt for spec in pmspecs for bin in bins ]
                return np.sum(all, axis=0)

        sfc_map(t, nc_d01, lambda nc: get_pm(nc, bins=[1]), 
                levels=range(0, 130, 10),
                ticks=range(0, 130, 20),
                label="UFP (ground level) [$\mu$g m$^{{-3}}$]", pois={}, 
                output_path=t.strftime(args.plot_fpath_pattern).format(plot='UFP_Europe'),
                extend='max')
        sfc_map(t, nc_d01, lambda nc: get_pm(nc, bins=[1,2,3]), 
                levels=range(0, 130, 10),
                ticks=range(0, 130, 20),
                label="PM$_{{2.5}}$ (ground level) [$\mu$g m$^{{-3}}$]", pois={}, 
                output_path=t.strftime(args.plot_fpath_pattern).format(plot='PM2_5_Europe'),
                extend='max')
        sfc_map(t, nc_d01, lambda nc: get_pm(nc, bins=[1,2,3,4]), 
                levels=range(0, 130, 10),
                ticks=range(0, 130, 20),
                label="PM$_{{10}}$ (ground level) [$\mu$g m$^{{-3}}$]", pois={}, 
                output_path=t.strftime(args.plot_fpath_pattern).format(plot='PM10_Europe'),
                extend='max')
        # SLP Europe
        output_path = t.strftime(args.plot_fpath_pattern).format(plot='SLP_Europe')
        prec_prev = slp_map(t, nc_d01, prec_prev, output_path)

        nc_d01.close()
